%QUICK_KALMAN - Original representation of the Blanchard et al (2013)
%information structure

function [out,GG0,GG1,GG2,F,sqQ,H,sqR] = quick_kalman(param)

sigtmp = param.sigtmp;
sige = sigtmp*(1-param.rho);
sign = sqrt(param.rho)*sigtmp;


sigv = param.sigv;
rhox = param.rho;
rhoz = param.rho;


%dx(t),z(t),z(t-1)
F          = zeros(4);
F(1,1)     = rhox;
F(2,2)     = rhoz;
F(3,2)     = 1;

H = [1 1 -1 0; 0 1 0 -1]';  %obs coeffs

%H = H(:,1);

%shocks to fundamental process
Q = zeros(3);
Q(1,1) = sige^2;
Q(2,2) = sign^2;
Q(4,4) = sigv^2;

%measurement errors
R = zeros(size(H,2)); 

P = eye(size(F,1));
crit = 1;
mm = 1;
while crit > 1e-13 && mm < 10000
    P_new = F*(P-P*H*((H'*P*H+R)\(H'*P)))*F' + Q;
    crit = max(max(abs(P-P_new)));
    P = .5*P + .5*(1/2)*(P_new+P_new');
    mm = mm+1;
end


%Depdendence on xi(t-1)
GG0 = P*H*((H'*P*H+R)\(H'*F));

%Depedence on xi(t-1,t-1)
GG1 = F + P*H*((H'*P*H+R)\(-H'*F));

%Depepdence on v(t), state shocks
GG2 = P*H*((H'*P*H+R)\(H'))*sqrt(Q);

%Depepdence on w(t), obs shocks
GG3 = P*H/(H'*P*H+R)*sqrt(R);

sqQ = sqrt(Q);
sqR = sqrt(R);


out = [GG0(:);GG1(:);GG2(:);GG3(:);F(:);sqQ(:)];
