%QUICK_KALMAN - Original representation of the Blanchard et al (2013)
%information structure

function [out,GG0,GG1,GG2,F,sqQ,H,sqR] = quick_kalman(param)

sige = param.sige;
sign = param.sign;
sigv = param.sigv;
rhox = param.rhox;
rhoz = param.rhoz;


%dx(t),z(t),z(t-1)
F          = zeros(4);
F(1,1)     = rhox;
F(2,2)     = rhoz;
F(3,2)     = 1;

H = [1 1 -1 0; 0 1 0 -1]';  %obs coeffs

%H = H(:,1);

%shocks to fundamental process
Q = zeros(3);
Q(1,1) = sige^2;
Q(2,2) = sign^2;
Q(4,4) = sigv^2;

%measurement errors
R = zeros(size(H,2)); 
P = eye(size(F,1));
crit = 1;
while crit > 1e-16
    P_new = F*(P-P*H*((H'*P*H+R)\(H'*P)))*F' + Q;
    crit = max(max(abs(P-P_new)));
    P = P_new;
end


%Depdendence on xi(t-1)
GG0 = P*H*((H'*P*H+R)\(H'*F));

%Depedence on xi(t-1,t-1)
GG1 = F + P*H*((H'*P*H+R)\(-H'*F));

%Depepdence on v(t), state shocks
GG2 = P*H*((H'*P*H+R)\(H'))*sqrt(Q);

%Depepdence on w(t), obs shocks
GG3 = P*H/(H'*P*H+R)*sqrt(R);

sqQ = sqrt(Q);
sqR = sqrt(R);


out = [GG0(:);GG1(:);GG2(:);GG3(:);F(:);sqQ(:)];





function out = obj(P,H,F,Q,R)

P = reshape(P,[5,5]);
out = 10000*vec(P-(F*(P-P*H*((H'*P*H+R)\(H'*P)))*F' + Q));